"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const URI = require("urijs");
class SyncedRealm {
    constructor(discovery, config) {
        this.opening = false;
        this.discovery = discovery;
        this.config = config;
        const self = this;
        this.realmPromise = new Promise((resolve, reject) => {
            self.realmResolve = resolve;
            self.realmReject = reject;
        });
        if (!this.config.sync) {
            throw new Error("cannot construct SyncedRealm without `sync` configuration");
        }
        const uri = new URI(this.config.sync.url);
        const syncLabel = uri.host();
        this.discovery.find("sync", ["master", syncLabel]).then((handle) => {
            if (handle) {
                this.reconfigure(handle);
            }
        });
        this.watch = discovery.watchService("sync", ["master", syncLabel]);
        this.watch.on("available", (handle) => {
            this.reconfigure(handle);
        });
    }
    reconfigure(handle) {
        const url = new URI(this.config.sync.url).host(handle.address).port(handle.port).toString();
        if (this.url === url) {
            return;
        }
        this.url = url;
        if (this.realm) {
            console.log(`realm ${this.config.path} reconnecting to ${handle.address}:${handle.port}`);
            const syncSession = this.realm.syncSession;
            if (syncSession) {
                syncSession._overrideServer(handle.address, handle.port);
            }
            else {
                this.realmPromise.then(() => {
                    this.realm.syncSession._overrideServer(handle.address, handle.port);
                }).catch((err) => {
                    console.log("Could set overrideServer on Realm", err);
                });
            }
        }
        else if (!this.opening) {
            this.opening = true;
            Realm.open({
                sync: {
                    error: (session, err) => {
                        this.realmReject(err);
                    },
                    user: this.config.sync.user,
                    url,
                },
                schema: this.config.schema,
                path: this.config.path,
            }).then((realm) => {
                this.opening = false;
                this.realm = realm;
                this.realmResolve(this.realm);
            }).catch((err) => {
                this.realmReject(err);
            });
        }
    }
    open() {
        return this.realmPromise;
    }
    close() {
        if (this.realm) {
            this.realm.close();
            delete this.realm;
        }
    }
}
exports.SyncedRealm = SyncedRealm;
//# sourceMappingURL=SyncedRealm.js.map